;;;; gltut01.lisp

(in-package #:gltut01)

(defvar *vertex-shader* "
#version 130
in vec4 position;
void main()
{
    gl_Position = position;
}
")

(defvar *fragment-shader* "
#version 130
out vec4 outputColor;
void main()
{
    outputColor = vec4(1.0f, 1.0f, 1.0f, 1.0f);
}
")

(defvar *the-program*)
(defvar *position-buffer-object*)
(defvar *vertex-array-object*)

(defun create-shader (shader-type shader-string)
  (let ((shader-id (gl:create-shader shader-type)))
    (gl:shader-source shader-id shader-string)
    (gl:compile-shader shader-id)
    (when (not (gl:get-shader shader-id :compile-status))
      (format t "~a~%" (gl:get-shader-info-log shader-id)))
    shader-id))

(defun create-program (shader-lst)
  (let ((program-id (gl:create-program)))
    (dolist (shader-id shader-lst)
      (gl:attach-shader program-id shader-id))
    (gl:bind-attrib-location program-id 0 "position")
    (gl:link-program program-id)
    (when (not (gl:get-program program-id :link-status))
      (format t "~a~%" (gl:get-program-info-log program-id)))
    program-id))

(defun init-program ()
  (let ((shader-lst (list (create-shader :vertex-shader *vertex-shader*)
			  (create-shader :fragment-shader *fragment-shader*))))
    (setf *the-program* (create-program shader-lst))
    (dolist (shader-id shader-lst)
      (gl:delete-shader shader-id))))

(defun init-vertex-buffer ()
  (setf *position-buffer-object* (elt (gl:gen-buffers 1) 0))
  (gl:bind-buffer :array-buffer *position-buffer-object*)
  (let ((arr (gl:alloc-gl-array :float 12))
	(verts #(0.75 0.75 0.0 1.0
		 0.75 -0.75 0.0 1.0
		 -0.75 -0.75 0.0 1.0)))
    (dotimes (i (length verts))
      (setf (gl:glaref arr i) (aref verts i)))
    (gl:buffer-data :array-buffer :static-draw arr)
    (gl:free-gl-array arr))
  (gl:bind-buffer :array-buffer 0))

(defun init (win gl-context)
  (sdl2:gl-make-current win gl-context)
  (init-program)
  (init-vertex-buffer)
  (setf *vertex-array-object* (gl:gen-vertex-array))
  (gl:bind-vertex-array *vertex-array-object*))


(defun display (win)
  (gl:clear-color 0.0 0.0 0.0 0.0)
  (gl:clear :color-buffer-bit)
  (gl:use-program *the-program*)
  (gl:bind-buffer :array-buffer *position-buffer-object*)
  (gl:enable-vertex-attrib-array 0)
  (gl:vertex-attrib-pointer 0 4 :float nil 0 (cffi:null-pointer))
  (gl:draw-arrays :triangles 0 3)
  (gl:disable-vertex-attrib-array 0)
  (gl:use-program 0)
  (sdl2:gl-swap-window win))

(defun hello-triangle ()
  (sdl2:with-init (:everything)
    (format t "Using SDL Library Version: ~D.~D.~D~%"
	    sdl2-ffi:+sdl-major-version+
	    sdl2-ffi:+sdl-minor-version+
	    sdl2-ffi:+sdl-patchlevel+)
    (finish-output)

    (sdl2:with-window (win :flags '(:shown :opengl))
      (sdl2:with-gl-context (gl-context win)
	(init win gl-context)
	(sdl2:with-event-loop (:method :poll)
	  (:idle
	   ()
	   (display win))

	  (:quit () t))))))
